import { CommonSession, dataTypeFrom, isTypedArray, SessionNodeData, SessionNodeInfo, SessionRunInputOption } from "../common";

export class MNNSession extends CommonSession {
	#session: any
	#inputs: Record<string, SessionNodeInfo> | null = null;
	#outputs: Record<string, SessionNodeInfo> | null = null;

	public constructor(modelData: Uint8Array) {
		super();
		const addon = require("../../../build/mnn.node")
		this.#session = new addon.MNNSession(modelData);
	}

	public run(inputs: Record<string, SessionNodeData | SessionRunInputOption>): Promise<Record<string, Float32Array>> {
		const inputArgs: Record<string, any> = {};
		for (const [name, option] of Object.entries(inputs)) {
			if (isTypedArray(option)) inputArgs[name] = { data: option }
			else inputArgs[name] = { ...option, type: option.type ? dataTypeFrom(option.type) : undefined };
		}
		return new Promise((resolve, reject) => this.#session.Run(inputArgs, (err: any, res: any) => {
			if (err) return reject(err);
			const result: Record<string, Float32Array> = {};
			for (const [name, val] of Object.entries(res)) result[name] = new Float32Array(val as ArrayBuffer);
			resolve(result);
		}))
	}
	public get inputs(): Record<string, SessionNodeInfo> { return this.#inputs ??= this.#session.GetInputsInfo(); }
	public get outputs(): Record<string, SessionNodeInfo> { return this.#outputs ??= this.#session.GetOutputsInfo(); }

}