
export enum ImreadModes {
	IMREAD_UNCHANGED = -1,
	IMREAD_GRAYSCALE = 0,
	IMREAD_COLOR_BGR = 1,
	IMREAD_COLOR = 1,
	IMREAD_ANYDEPTH = 2,
	IMREAD_ANYCOLOR = 4,
	IMREAD_LOAD_GDAL = 8,
	IMREAD_REDUCED_GRAYSCALE_2 = 16,
	IMREAD_REDUCED_COLOR_2 = 17,
	IMREAD_REDUCED_GRAYSCALE_4 = 32,
	IMREAD_REDUCED_COLOR_4 = 33,
	IMREAD_REDUCED_GRAYSCALE_8 = 64,
	IMREAD_REDUCED_COLOR_8 = 65,
	IMREAD_IGNORE_ORIENTATION = 128,
	IMREAD_COLOR_RGB = 256,
};

interface MatConstructorOption {
	mode?: ImreadModes;
}

export class Mat {
	#mat: any

	public static async load(image: string, option?: MatConstructorOption) {
		let buffer: Uint8Array
		if (/^https?:\/\//.test(image)) buffer = await fetch(image).then(res => res.arrayBuffer()).then(res => new Uint8Array(res));
		else buffer = await import("fs").then(fs => fs.promises.readFile(image));
		return new Mat(buffer, option);
	}

	public constructor(imageData: Uint8Array, option?: MatConstructorOption) {
		const addon = require("../../build/cv.node");
		if ((imageData as any) instanceof addon.Mat) this.#mat = imageData;
		else this.#mat = new addon.Mat(imageData, option);
	}

	public get empty(): boolean { return this.#mat.IsEmpty() }

	public get cols(): number { return this.#mat.GetCols(); }

	public get rows(): number { return this.#mat.GetRows(); }

	public get width() { return this.cols; }

	public get height() { return this.rows; }

	public get channels() { return this.#mat.GetChannels(); }

	public resize(width: number, height: number) { return new Mat(this.#mat.Resize.bind(this.#mat)(width, height)); }

	public crop(sx: number, sy: number, sw: number, sh: number) { return new Mat(this.#mat.Crop(sx, sy, sw, sh)); }

	public rotate(sx: number, sy: number, angleDeg: number) { return new Mat(this.#mat.Rotate(sx, sy, angleDeg)); }

	public get data() { return new Uint8Array(this.#mat.Data()); }

	public encode(extname: string) { return new Uint8Array(this.#mat.Encode({ extname })); }

	public clone() { return new Mat(this.#mat.Clone()); }

	public circle(x: number, y: number, radius: number, options?: {
		color?: { r: number, g: number, b: number },
		thickness?: number
		lineType?: number
	}) {
		this.#mat.DrawCircle(
			x, y, radius,
			options?.color?.b ?? 0,
			options?.color?.g ?? 0,
			options?.color?.r ?? 0,
			options?.thickness ?? 1,
			options?.lineType ?? 8,
			0,
		);
	}
}
