import { Mat } from "../../cv/mat";
import { convertImage } from "../common/processors";
import { FaceRecognition, FaceRecognitionPredictOption } from "./common";

const MODEL_URL_CONFIG = {
	MOBILEFACENET_ADAFACE_ONNX: `https://www.modelscope.cn/models/luyizhi/basic_cv/resolve/master/faceid/mobilefacenet_adaface.onnx`,
	MOBILEFACENET_ADAFACE_MNN: `https://www.modelscope.cn/models/luyizhi/basic_cv/resolve/master/faceid/mobilefacenet_adaface.mnn`,
};

export class AdaFace extends FaceRecognition {

	public static async load(type?: keyof typeof MODEL_URL_CONFIG) {
		return this.cacheModel(MODEL_URL_CONFIG[type ?? "MOBILEFACENET_ADAFACE_ONNX"], { createModel: true }).then(r => r.model);
	}

	public async doPredict(image: Mat, option?: FaceRecognitionPredictOption): Promise<number[]> {
		const input = this.input;
		const output = this.output;

		if (option?.crop) image = image.crop(option.crop.sx, option.crop.sy, option.crop.sw, option.crop.sh);
		image = image.resize(input.shape[3], input.shape[2]);

		const nchwImageData = convertImage(image.data, { sourceImageFormat: "bgr", targetColorFormat: "rgb", targetShapeFormat: "nchw", targetNormalize: { mean: [127.5], std: [127.5] } });

		const embedding = await this.session.run({
			[input.name]: {
				type: "float32",
				data: nchwImageData,
				shape: [1, 3, input.shape[2], input.shape[3]],
			}
		}).then(res => res[output.name]);

		return new Array(...embedding);
	}

}
